@@ -45,7 +45,9 @@ extern gint flexi_servers;
 extern gint xdmcp_pending;
 extern GSList *displays;
 extern GdmConnection *fifoconn;
+extern GdmConnection *pipeconn;
 extern GdmConnection *unixconn;
+extern int slave_fifo_pipe_fd; /* the slavepipe (like fifo) connection, this is the write end */
 
 static gboolean
 gdm_display_check_loop (GdmDisplay *disp)
@@ -227,13 +229,15 @@ gdm_display_manage (GdmDisplay *d)
 
 	gdm_connection_close (fifoconn);
 	fifoconn = NULL;
+	gdm_connection_close (pipeconn);
+	pipeconn = NULL;
 	gdm_connection_close (unixconn);
 	unixconn = NULL;
 
 	closelog ();
 
 	/* Close everything */
-	gdm_close_all_descriptors (0 /* from */, fds[0] /* except */);
+	gdm_close_all_descriptors (0 /* from */, fds[0] /* except */, slave_fifo_pipe_fd /* except2 */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
