@@ -69,7 +69,8 @@ static void gdm_restart_now (void);
 GSList *displays = NULL;	/* List of displays managed */
 GSList *xservers = NULL;	/* List of x server definitions */
 gint high_display_num = 0;	/* Highest local non-flexi display */
-gint sessions = 0;		/* Number of remote sessions */
+gint xdmcp_sessions = 0;	/* Number of remote sessions */
+gint xdmcp_pending = 0;		/* Number of pending remote sessions */
 gint flexi_servers = 0;		/* Number of flexi servers */
 uid_t GdmUserId;		/* Userid under which gdm should run */
 gid_t GdmGroupId;		/* Groupid under which gdm should run */
@@ -81,7 +82,9 @@ gboolean preserve_ld_vars = FALSE; /* Preserve the ld environment variables */
 gboolean no_daemon = FALSE;	/* Do not daemonize */
 
 GdmConnection *fifoconn = NULL; /* Fifo connection */
+GdmConnection *pipeconn = NULL; /* slavepipe (handled just like Fifo for compatibility) connection */
 GdmConnection *unixconn = NULL; /* UNIX Socket connection */
+int slave_fifo_pipe_fd = -1; /* the slavepipe connection */
 
 char *gdm_charset = NULL;
 
@@ -105,10 +108,6 @@ gchar *GdmGroup = NULL;
 gchar *GdmSessDir = NULL;
 gchar *GdmXsession = NULL;
 gchar *GdmLocaleFile = NULL;
-#if 0
-/* FIXME: Maybe just whack this */
-gchar *GdmGnomeDefaultSession = NULL;
-#endif
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
 gboolean GdmAlwaysRestartServer = FALSE;
@@ -180,7 +179,6 @@ gboolean GdmDisallowTCP = TRUE;
 /* set in the main function */
 char **stored_argv = NULL;
 int stored_argc = 0;
-char *stored_path = NULL;
 
 static time_t config_file_mtime = 0;
 
@@ -223,12 +221,12 @@ compare_displays (gconstpointer a, gconstpointer b)
 static void 
 gdm_config_parse (void)
 {
-    gchar *k, *v;
-    void *iter;
     struct passwd *pwent;
     struct group *grent;
     struct stat statbuf;
     gchar *bin;
+    VeConfig *cfg;
+    GList *list, *li;
     
     displays = NULL;
     high_display_num = 0;
@@ -241,43 +239,43 @@ gdm_config_parse (void)
     }
 
     /* Parse configuration options */
-    gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+    cfg = ve_config_new (GDM_CONFIG_FILE);
 
-    GdmChooser = gnome_config_get_string (GDM_KEY_CHOOSER);
-    GdmDefaultPath = gnome_config_get_string (GDM_KEY_PATH);
-    GdmDisplayInit = gnome_config_get_string (GDM_KEY_INITDIR);
-    GdmAutomaticLoginEnable = gnome_config_get_bool (GDM_KEY_AUTOMATICLOGIN_ENABLE);
-    GdmAutomaticLogin = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
-    GdmAlwaysRestartServer = gnome_config_get_bool (GDM_KEY_ALWAYSRESTARTSERVER);
-    GdmGreeter = gnome_config_get_string (GDM_KEY_GREETER);
-    GdmRemoteGreeter = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
-    GdmAddGtkModules = gnome_config_get_bool (GDM_KEY_ADD_GTK_MODULES);
-    GdmDoubleLoginWarning = gnome_config_get_bool (GDM_KEY_DOUBLELOGINWARNING);
-    GdmGtkModulesList = gnome_config_get_string (GDM_KEY_GTK_MODULES_LIST);	
-    GdmGroup = gnome_config_get_string (GDM_KEY_GROUP);
-    GdmHalt = gnome_config_get_string (GDM_KEY_HALT);
-    GdmKillInitClients = gnome_config_get_bool (GDM_KEY_KILLIC);
-    GdmLogDir= gnome_config_get_string (GDM_KEY_LOGDIR);
-    GdmPidFile = gnome_config_get_string (GDM_KEY_PIDFILE);
-    GdmSessionDir = gnome_config_get_string (GDM_KEY_SESSDIR);
-    GdmPostLogin = gnome_config_get_string (GDM_KEY_POSTLOGIN);
-    GdmPreSession = gnome_config_get_string (GDM_KEY_PRESESS);
-    GdmPostSession = gnome_config_get_string (GDM_KEY_POSTSESS);
-    GdmFailsafeXServer = gnome_config_get_string (GDM_KEY_FAILSAFE_XSERVER);
-    GdmXKeepsCrashing = gnome_config_get_string (GDM_KEY_XKEEPSCRASHING);
-    GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
-    GdmConfigAvailable = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
-    GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
-    GdmChooserButton = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
-    GdmBrowser = gnome_config_get_bool (GDM_KEY_BROWSER);
-    GdmGlobalFaceDir = gnome_config_get_string (GDM_KEY_FACEDIR);
-    GdmXineramaScreen = gnome_config_get_int (GDM_KEY_XINERAMASCREEN);
-    GdmReboot = gnome_config_get_string (GDM_KEY_REBOOT);
-    GdmRetryDelay = gnome_config_get_int (GDM_KEY_RETRYDELAY);
-    GdmRootPath = gnome_config_get_string (GDM_KEY_ROOTPATH);
-    GdmServAuthDir = gnome_config_get_string (GDM_KEY_SERVAUTH);
-    GdmSessDir = gnome_config_get_string (GDM_KEY_SESSDIR);
-    GdmXsession = gnome_config_get_string (GDM_KEY_BASEXSESSION);
+    GdmChooser = ve_config_get_string (cfg, GDM_KEY_CHOOSER);
+    GdmDefaultPath = ve_config_get_string (cfg, GDM_KEY_PATH);
+    GdmDisplayInit = ve_config_get_string (cfg, GDM_KEY_INITDIR);
+    GdmAutomaticLoginEnable = ve_config_get_bool (cfg, GDM_KEY_AUTOMATICLOGIN_ENABLE);
+    GdmAutomaticLogin = ve_config_get_string (cfg, GDM_KEY_AUTOMATICLOGIN);
+    GdmAlwaysRestartServer = ve_config_get_bool (cfg, GDM_KEY_ALWAYSRESTARTSERVER);
+    GdmGreeter = ve_config_get_string (cfg, GDM_KEY_GREETER);
+    GdmRemoteGreeter = ve_config_get_string (cfg, GDM_KEY_REMOTEGREETER);
+    GdmAddGtkModules = ve_config_get_bool (cfg, GDM_KEY_ADD_GTK_MODULES);
+    GdmDoubleLoginWarning = ve_config_get_bool (cfg, GDM_KEY_DOUBLELOGINWARNING);
+    GdmGtkModulesList = ve_config_get_string (cfg, GDM_KEY_GTK_MODULES_LIST);	
+    GdmGroup = ve_config_get_string (cfg, GDM_KEY_GROUP);
+    GdmHalt = ve_config_get_string (cfg, GDM_KEY_HALT);
+    GdmKillInitClients = ve_config_get_bool (cfg, GDM_KEY_KILLIC);
+    GdmLogDir= ve_config_get_string (cfg, GDM_KEY_LOGDIR);
+    GdmPidFile = ve_config_get_string (cfg, GDM_KEY_PIDFILE);
+    GdmSessionDir = ve_config_get_string (cfg, GDM_KEY_SESSDIR);
+    GdmPostLogin = ve_config_get_string (cfg, GDM_KEY_POSTLOGIN);
+    GdmPreSession = ve_config_get_string (cfg, GDM_KEY_PRESESS);
+    GdmPostSession = ve_config_get_string (cfg, GDM_KEY_POSTSESS);
+    GdmFailsafeXServer = ve_config_get_string (cfg, GDM_KEY_FAILSAFE_XSERVER);
+    GdmXKeepsCrashing = ve_config_get_string (cfg, GDM_KEY_XKEEPSCRASHING);
+    GdmConfigurator = ve_config_get_string (cfg, GDM_KEY_CONFIGURATOR);
+    GdmConfigAvailable = ve_config_get_bool (cfg, GDM_KEY_CONFIG_AVAILABLE);
+    GdmSystemMenu = ve_config_get_bool (cfg, GDM_KEY_SYSMENU);
+    GdmChooserButton = ve_config_get_bool (cfg, GDM_KEY_CHOOSER_BUTTON);
+    GdmBrowser = ve_config_get_bool (cfg, GDM_KEY_BROWSER);
+    GdmGlobalFaceDir = ve_config_get_string (cfg, GDM_KEY_FACEDIR);
+    GdmXineramaScreen = ve_config_get_int (cfg, GDM_KEY_XINERAMASCREEN);
+    GdmReboot = ve_config_get_string (cfg, GDM_KEY_REBOOT);
+    GdmRetryDelay = ve_config_get_int (cfg, GDM_KEY_RETRYDELAY);
+    GdmRootPath = ve_config_get_string (cfg, GDM_KEY_ROOTPATH);
+    GdmServAuthDir = ve_config_get_string (cfg, GDM_KEY_SERVAUTH);
+    GdmSessDir = ve_config_get_string (cfg, GDM_KEY_SESSDIR);
+    GdmXsession = ve_config_get_string (cfg, GDM_KEY_BASEXSESSION);
     if (ve_string_empty (GdmXsession)) {
 	    gdm_info (_("%s: BaseXsession empty, using %s/gdm/Xsession"),
 			"gdm_config_parse",
@@ -286,41 +284,37 @@ gdm_config_parse (void)
 	    GdmXsession = g_strconcat (EXPANDED_SYSCONFDIR, "/gdm/Xsession",
 				       NULL);
     }
-    GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
-    GdmLocaleFile = gnome_config_get_string (GDM_KEY_LOCFILE);
-#if 0
-/* FIXME: Maybe just whack this */
-    GdmGnomeDefaultSession = gnome_config_get_string (GDM_KEY_GNOMEDEFAULTSESSION);
-#endif
-    GdmUser = gnome_config_get_string (GDM_KEY_USER);
-    GdmUserAuthDir = gnome_config_get_string (GDM_KEY_UAUTHDIR);
-    GdmUserAuthFile = gnome_config_get_string (GDM_KEY_UAUTHFILE);
-    GdmUserAuthFB = gnome_config_get_string (GDM_KEY_UAUTHFB);
+    GdmSuspend = ve_config_get_string (cfg, GDM_KEY_SUSPEND);
+    GdmLocaleFile = ve_config_get_string (cfg, GDM_KEY_LOCFILE);
+    GdmUser = ve_config_get_string (cfg, GDM_KEY_USER);
+    GdmUserAuthDir = ve_config_get_string (cfg, GDM_KEY_UAUTHDIR);
+    GdmUserAuthFile = ve_config_get_string (cfg, GDM_KEY_UAUTHFILE);
+    GdmUserAuthFB = ve_config_get_string (cfg, GDM_KEY_UAUTHFB);
 
-    GdmTimedLoginEnable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
-    GdmTimedLogin = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
-    GdmTimedLoginDelay = gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+    GdmTimedLoginEnable = ve_config_get_bool (cfg, GDM_KEY_TIMED_LOGIN_ENABLE);
+    GdmTimedLogin = ve_config_get_string (cfg, GDM_KEY_TIMED_LOGIN);
+    GdmTimedLoginDelay = ve_config_get_int (cfg, GDM_KEY_TIMED_LOGIN_DELAY);
 
-    GdmAllowRoot = gnome_config_get_bool (GDM_KEY_ALLOWROOT);
-    GdmAllowRemoteRoot = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEROOT);
-    GdmAllowRemoteAutoLogin = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEAUTOLOGIN);
-    GdmRelaxPerms = gnome_config_get_int (GDM_KEY_RELAXPERM);
-    GdmUserMaxFile = gnome_config_get_int (GDM_KEY_MAXFILE);
-    GdmSessionMaxFile = gnome_config_get_int (GDM_KEY_SESSIONMAXFILE);
+    GdmAllowRoot = ve_config_get_bool (cfg, GDM_KEY_ALLOWROOT);
+    GdmAllowRemoteRoot = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEROOT);
+    GdmAllowRemoteAutoLogin = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEAUTOLOGIN);
+    GdmRelaxPerms = ve_config_get_int (cfg, GDM_KEY_RELAXPERM);
+    GdmUserMaxFile = ve_config_get_int (cfg, GDM_KEY_MAXFILE);
+    GdmSessionMaxFile = ve_config_get_int (cfg, GDM_KEY_SESSIONMAXFILE);
 
-    GdmXdmcp = gnome_config_get_bool (GDM_KEY_XDMCP);
-    GdmDispPerHost = gnome_config_get_int (GDM_KEY_DISPERHOST);
-    GdmMaxPending = gnome_config_get_int (GDM_KEY_MAXPEND);
-    GdmMaxManageWait = gnome_config_get_int (GDM_KEY_MAXWAIT);
-    GdmMaxSessions = gnome_config_get_int (GDM_KEY_MAXSESS);
-    GdmPort = gnome_config_get_int (GDM_KEY_UDPPORT);
-    GdmIndirect = gnome_config_get_bool (GDM_KEY_INDIRECT);
-    GdmMaxIndirect = gnome_config_get_int (GDM_KEY_MAXINDIR);
-    GdmMaxIndirectWait = gnome_config_get_int (GDM_KEY_MAXINDWAIT);    
-    GdmPingInterval = gnome_config_get_int (GDM_KEY_PINGINTERVAL);    
-    GdmWilling = gnome_config_get_string (GDM_KEY_WILLING);    
+    GdmXdmcp = ve_config_get_bool (cfg, GDM_KEY_XDMCP);
+    GdmDispPerHost = ve_config_get_int (cfg, GDM_KEY_DISPERHOST);
+    GdmMaxPending = ve_config_get_int (cfg, GDM_KEY_MAXPEND);
+    GdmMaxManageWait = ve_config_get_int (cfg, GDM_KEY_MAXWAIT);
+    GdmMaxSessions = ve_config_get_int (cfg, GDM_KEY_MAXSESS);
+    GdmPort = ve_config_get_int (cfg, GDM_KEY_UDPPORT);
+    GdmIndirect = ve_config_get_bool (cfg, GDM_KEY_INDIRECT);
+    GdmMaxIndirect = ve_config_get_int (cfg, GDM_KEY_MAXINDIR);
+    GdmMaxIndirectWait = ve_config_get_int (cfg, GDM_KEY_MAXINDWAIT);    
+    GdmPingInterval = ve_config_get_int (cfg, GDM_KEY_PINGINTERVAL);    
+    GdmWilling = ve_config_get_string (cfg, GDM_KEY_WILLING);    
 
-    GdmStandardXServer = gnome_config_get_string (GDM_KEY_STANDARD_XSERVER);    
+    GdmStandardXServer = ve_config_get_string (cfg, GDM_KEY_STANDARD_XSERVER);    
     if (ve_string_empty (GdmStandardXServer) ||
 	access (GdmStandardXServer, X_OK) != 0) {
 	    gdm_info (_("%s: Standard X server not found, trying alternatives"),
@@ -336,18 +330,16 @@ gdm_config_parse (void)
 		    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
 	    }
     }
-    GdmFlexibleXServers = gnome_config_get_int (GDM_KEY_FLEXIBLE_XSERVERS);    
-    GdmXnest = gnome_config_get_string (GDM_KEY_XNEST);    
+    GdmFlexibleXServers = ve_config_get_int (cfg, GDM_KEY_FLEXIBLE_XSERVERS);    
+    GdmXnest = ve_config_get_string (cfg, GDM_KEY_XNEST);    
     if (ve_string_empty (GdmXnest))
 	    GdmXnest = NULL;
 
-    GdmFirstVT = gnome_config_get_int (GDM_KEY_FIRSTVT);    
-    GdmVTAllocation = gnome_config_get_bool (GDM_KEY_VTALLOCATION);    
-    GdmDisallowTCP = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);    
+    GdmFirstVT = ve_config_get_int (cfg, GDM_KEY_FIRSTVT);    
+    GdmVTAllocation = ve_config_get_bool (cfg, GDM_KEY_VTALLOCATION);    
+    GdmDisallowTCP = ve_config_get_bool (cfg, GDM_KEY_DISALLOWTCP);    
 
-    GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
-
-    gnome_config_pop_prefix();
+    GdmDebug = ve_config_get_bool (cfg, GDM_KEY_DEBUG);
 
     /* sanitize some values */
 
@@ -365,7 +357,7 @@ gdm_config_parse (void)
     }
 
     if (GdmAutomaticLogin != NULL &&
-	strcmp (GdmAutomaticLogin, "root") == 0) {
+	strcmp (GdmAutomaticLogin, gdm_root_user ()) == 0) {
 	    gdm_info (_("%s: Root cannot be autologged in, turing off automatic login"), "gdm_config_parse");
 	    g_free (GdmAutomaticLogin);
 	    GdmAutomaticLogin = NULL;
@@ -378,7 +370,7 @@ gdm_config_parse (void)
     }
 
     if (GdmTimedLogin != NULL &&
-	strcmp (GdmTimedLogin, "root") == 0) {
+	strcmp (GdmTimedLogin, gdm_root_user ()) == 0) {
 	    gdm_info (_("%s: Root cannot be autologged in, turing off timed login"), "gdm_config_parse");
 	    g_free (GdmTimedLogin);
 	    GdmTimedLogin = NULL;
@@ -415,29 +407,26 @@ gdm_config_parse (void)
 	gdm_error (_("%s: No sessions directory specified."), "gdm_config_parse");
 
     /* Find server definitions */
-    iter = gnome_config_init_iterator_sections ("=" GDM_CONFIG_FILE "=/");
-    iter = gnome_config_iterator_next (iter, &k, NULL);
-    
-    while (iter) {
-	    if (strncmp (k, "server-", strlen ("server-")) == 0) {
-		    char *section;
+    list = ve_config_get_sections (cfg);
+    for (li = list; li != NULL; li = li->next) {
+	    const char *sec = li->data;
+	    if (strncmp (sec, "server-", strlen ("server-")) == 0) {
 		    GdmXServer *svr = g_new0 (GdmXServer, 1);
+		    char buf[256];
 
-		    section = g_strdup_printf ("=" GDM_CONFIG_FILE "=/%s/", k);
-		    gnome_config_push_prefix (section);
-
-		    svr->id = g_strdup (k + strlen ("server-"));
-		    svr->name = gnome_config_get_string (GDM_KEY_SERVER_NAME);
-		    svr->command = gnome_config_get_string
-			    (GDM_KEY_SERVER_COMMAND);
-		    svr->flexible = gnome_config_get_bool
-			    (GDM_KEY_SERVER_FLEXIBLE);
-		    svr->choosable = gnome_config_get_bool
-			    (GDM_KEY_SERVER_CHOOSABLE);
-		    svr->handled = gnome_config_get_bool
-			    (GDM_KEY_SERVER_HANDLED);
-		    svr->chooser = gnome_config_get_bool
-			    (GDM_KEY_SERVER_CHOOSER);
+		    svr->id = g_strdup (sec + strlen ("server-"));
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_NAME, sec);
+		    svr->name = ve_config_get_string (cfg, buf);
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_COMMAND, sec);
+		    svr->command = ve_config_get_string (cfg, buf);
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_FLEXIBLE, sec);
+		    svr->flexible = ve_config_get_bool (cfg, buf);
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_CHOOSABLE, sec);
+		    svr->choosable = ve_config_get_bool (cfg, buf);
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_HANDLED, sec);
+		    svr->handled = ve_config_get_bool (cfg, buf);
+		    g_snprintf (buf, sizeof (buf), "%s/" GDM_KEY_SERVER_CHOOSER, sec);
+		    svr->chooser = ve_config_get_bool (cfg, buf);
 
 		    if (ve_string_empty (svr->command)) {
 			    gdm_error (_("%s: Empty server command, "
@@ -447,16 +436,10 @@ gdm_config_parse (void)
 			    svr->command = g_strdup (GdmStandardXServer);
 		    }
 
-		    g_free (section);
-		    gnome_config_pop_prefix ();
-
 		    xservers = g_slist_append (xservers, svr);
 	    }
-
-	    g_free (k);
-
-	    iter = gnome_config_iterator_next (iter, &k, NULL);
     }
+    ve_config_free_list_of_strings (list);
 
     if (xservers == NULL ||
 	gdm_find_x_server (GDM_STANDARD) == NULL) {
@@ -473,30 +456,33 @@ gdm_config_parse (void)
     }
 
     /* Find local X server definitions */
-    iter = gnome_config_init_iterator ("=" GDM_CONFIG_FILE "=/" GDM_KEY_SERVERS);
-    iter = gnome_config_iterator_next (iter, &k, &v);
-    
-    while (iter) {
-	    if (isdigit (*k)) {
-		    int disp_num = atoi (k);
+    list = ve_config_get_keys (cfg, GDM_KEY_SERVERS);
+    for (li = list; li != NULL; li = li->next) {
+	    const char *key = li->data;
+	    if (isdigit (*key)) {
+		    char *full;
+		    char *val;
+		    int disp_num = atoi (key);
 		    GdmDisplay *disp;
 
 		    while (display_exists (disp_num)) {
 			    disp_num++;
 		    }
 
-		    if (disp_num != atoi (k)) {
+		    if (disp_num != atoi (key)) {
 			    gdm_error (_("%s: Display number %d in use!  I will use %d"),
-				       "gdm_config_parse", atoi (k), disp_num);
+				       "gdm_config_parse", atoi (key), disp_num);
 		    }
 
-		    disp = gdm_server_alloc (disp_num, v);
-		    if (disp == NULL) {
-			    g_free (k);
-			    g_free (v);
-			    iter = gnome_config_iterator_next (iter, &k, &v);
+		    full = g_strdup_printf ("%s/%s", GDM_KEY_SERVERS, key);
+		    val = ve_config_get_string (cfg, full);
+		    g_free (full);
+
+		    disp = gdm_server_alloc (disp_num, val);
+		    g_free (val);
+
+		    if (disp == NULL)
 			    continue;
-		    }
 		    displays = g_slist_insert_sorted (displays,
 						      disp,
 						      compare_displays);
@@ -505,11 +491,8 @@ gdm_config_parse (void)
 	    } else {
 		    gdm_info (_("%s: Invalid server line in config file. Ignoring!"), "gdm_config_parse");
 	    }
-	    g_free (k);
-	    g_free (v);
-
-	    iter = gnome_config_iterator_next (iter, &k, &v);
     }
+    ve_config_free_list_of_strings (list);
 
     if (displays == NULL && ! GdmXdmcp) {
 	    char *server = NULL;
@@ -594,7 +577,7 @@ gdm_config_parse (void)
 	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
-	    pwent = getpwnam (GdmUser);
+	    grent = getgrnam (GdmGroup);
     }
 
     if (grent == NULL) {
@@ -722,6 +705,8 @@ gdm_config_parse (void)
 
     /* Check that user authentication is properly configured */
     gdm_verify_check ();
+
+    ve_config_destroy (cfg);
 }
 
 /* If id == NULL, then get the first X server */
@@ -759,7 +744,7 @@ gdm_daemonify (void)
     pid = fork ();
     if (pid > 0) {
 
-	if ((pf = fopen (GdmPidFile, "w")) != NULL) {
+	if ((pf = gdm_safe_fopen_w (GdmPidFile)) != NULL) {
 	    fprintf (pf, "%d\n", (int)pid);
 	    fclose (pf);
 	}
@@ -869,7 +854,8 @@ gdm_final_cleanup (void)
 
 	/* Close stuff */
 
-	gdm_xdmcp_close ();
+	if (GdmXdmcp)
+		gdm_xdmcp_close ();
 
 	if (fifoconn != NULL) {
 		char *path;
@@ -880,6 +866,16 @@ gdm_final_cleanup (void)
 		fifoconn = NULL;
 	}
 
+	if (pipeconn != NULL) {
+		gdm_connection_close (pipeconn);
+		pipeconn = NULL;
+	}
+
+	if (slave_fifo_pipe_fd >= 0) {
+		close (slave_fifo_pipe_fd);
+		slave_fifo_pipe_fd = -1;
+	}
+
 	if (unixconn != NULL) {
 		gdm_connection_close (unixconn);
 		unlink (GDM_SUP_SOCKET);
@@ -933,7 +929,7 @@ deal_with_x_crashes (GdmDisplay *d)
 
 		    closelog ();
 
-		    gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+		    gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */);
 
 		    /* No error checking here - if it's messed the best response
 		    * is to ignore & try to continue */
@@ -944,21 +940,23 @@ deal_with_x_crashes (GdmDisplay *d)
 		    argv[0] = GdmXKeepsCrashing;
 		    argv[1] = NULL;
 
+		    gdm_restoreenv ();
+
 		    /* unset DISPLAY and XAUTHORITY if they exist
 		     * so that gdialog (if used) doesn't get confused */
-		    gnome_unsetenv ("DISPLAY");
-		    gnome_unsetenv ("XAUTHORITY");
+		    ve_unsetenv ("DISPLAY");
+		    ve_unsetenv ("XAUTHORITY");
 
 		    /* some promised variables */
-		    gnome_setenv ("XLOG", xlog, TRUE);
-		    gnome_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
-		    gnome_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
-		    gnome_setenv ("LIBEXECDIR", EXPANDED_LIBEXECDIR, TRUE);
-		    gnome_setenv ("SYSCONFDIR", EXPANDED_SYSCONFDIR, TRUE);
+		    ve_setenv ("XLOG", xlog, TRUE);
+		    ve_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
+		    ve_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
+		    ve_setenv ("LIBEXECDIR", EXPANDED_LIBEXECDIR, TRUE);
+		    ve_setenv ("SYSCONFDIR", EXPANDED_SYSCONFDIR, TRUE);
 
 		    /* To enable gettext stuff in the script */
-		    gnome_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
-		    gnome_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
+		    ve_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
+		    ve_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
 
 		    execv (argv[0], argv);
 	
@@ -1020,6 +1018,8 @@ suspend_machine (void)
 	if (GdmSuspendReal != NULL &&
 	    fork () == 0) {
 		char **argv;
+		if (GdmXdmcp)
+			gdm_xdmcp_close ();
 		/* In the child setup empty mask and set all signals to
 		 * default values */
 		gdm_unset_signals ();
@@ -1100,25 +1100,24 @@ gdm_cleanup_children (void)
 	    return TRUE;
 
     if (crashed) {
-	    gdm_debug ("gdm_cleanup_children: Slave crashed, killing it's "
+	    gdm_debug ("gdm_cleanup_children: Slave crashed, killing its "
 		       "children");
 
-	    if (d->sesspid > 1 &&
-		kill (d->sesspid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->sesspid, NULL, 0);
+	    if (d->sesspid > 1)
+		    kill (-(d->sesspid), SIGTERM);
 	    d->sesspid = 0;
-	    if (d->greetpid > 1 &&
-		kill (d->greetpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->greetpid, NULL, 0);
+	    if (d->greetpid > 1)
+		    kill (-(d->greetpid), SIGTERM);
 	    d->greetpid = 0;
-	    if (d->chooserpid > 1 &&
-		kill (d->chooserpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->chooserpid, NULL, 0);
+	    if (d->chooserpid > 1)
+		    kill (-(d->chooserpid), SIGTERM);
 	    d->chooserpid = 0;
-	    if (d->servpid > 1 &&
-		kill (d->servpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->servpid, NULL, 0);
+	    if (d->servpid > 1)
+		    kill (d->servpid, SIGTERM);
 	    d->servpid = 0;
+
+	    /* race avoider */
+	    sleep (1);
     }
 
     /* null all these, they are not valid most definately */
@@ -1171,6 +1170,18 @@ gdm_cleanup_children (void)
 	    status = DISPLAY_REMANAGE;
     }
 
+    if (status == DISPLAY_GREETERFAILED) {
+	    if (d->managetime + 10 >= time (NULL)) {
+		    d->try_different_greeter = TRUE;
+	    } else {
+		    d->try_different_greeter = FALSE;
+	    }
+	    /* now just remanage */
+	    status = DISPLAY_REMANAGE;
+    } else {
+	    d->try_different_greeter = FALSE;
+    }
+
     /* checkout if we can actually do stuff */
     switch (status) {
     case DISPLAY_REBOOT:
@@ -1353,8 +1364,6 @@ start_autopsy:
     if (gdm_restart_mode)
 	    gdm_safe_restart ();
 
-    g_main_loop_quit (main_loop);
-
     return TRUE;
 }
 
@@ -1363,8 +1372,7 @@ gdm_restart_now (void)
 {
 	gdm_info (_("GDM restarting ..."));
 	gdm_final_cleanup ();
-	if (stored_path != NULL)
-		gnome_setenv ("PATH", stored_path, TRUE);
+	gdm_restoreenv ();
 	execvp (stored_argv[0], stored_argv);
 	gdm_error (_("Failed to restart self"));
 	_exit (1);
@@ -1428,8 +1436,6 @@ store_argv (int argc, char *argv[])
 {
 	int i;
 
-	stored_path = g_strdup (g_getenv ("PATH"));
-
 	stored_argv = g_new0 (char *, argc + 1);
 	for (i = 0; i < argc; i++)
 		stored_argv[i] = g_strdup (argv[i]);
@@ -1437,28 +1443,67 @@ store_argv (int argc, char *argv[])
 	stored_argc = argc;
 }
 
+static void
+close_notify (gpointer data)
+{
+	GdmConnection **conn = data;
+	* conn = NULL;
+}
+
 static void
 create_connections (void)
 {
+	int p[2];
 	gchar *path;
 
 	path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
 	fifoconn = gdm_connection_open_fifo (path, 0660);
 	g_free (path);
 
-	if (fifoconn != NULL)
+	if (fifoconn != NULL) {
 		gdm_connection_set_handler (fifoconn,
 					    gdm_handle_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (fifoconn,
+						 &fifoconn,
+						 close_notify);
+	}
+
+	if (pipe (p) < 0) {
+		slave_fifo_pipe_fd = -1;
+		pipeconn = NULL;
+	} else {
+		slave_fifo_pipe_fd = p[1];
+		pipeconn = gdm_connection_open_fd (p[0]);
+	}
+
+	if (pipeconn != NULL) {
+		gdm_connection_set_handler (pipeconn,
+					    gdm_handle_message,
+					    NULL /* data */,
+					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (pipeconn,
+						 &pipeconn,
+						 close_notify);
+	} else {
+		close (p[0]);
+		close (p[1]);
+		slave_fifo_pipe_fd = -1;
+	}
+
 
 	unixconn = gdm_connection_open_unix (GDM_SUP_SOCKET, 0666);
 
-	if (unixconn != NULL)
+	if (unixconn != NULL) {
 		gdm_connection_set_handler (unixconn,
 					    gdm_handle_user_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (unixconn,
+						 &unixconn,
+						 close_notify);
+	}
 }
 
 struct poptOption options [] = {
@@ -1589,6 +1634,9 @@ main (int argc, char *argv[])
 	    gdm_charset = g_strdup (charset);
     }
 
+    /* get the name of the root user */
+    gdm_root_user ();
+
     /* Parse configuration file */
     gdm_config_parse();
 
@@ -1618,7 +1666,7 @@ main (int argc, char *argv[])
     if (no_daemon || getppid() == 1) {
 
 	/* Write pid to pidfile */
-	if ((pf = fopen (GdmPidFile, "w")) != NULL) {
+	if ((pf = gdm_safe_fopen_w (GdmPidFile)) != NULL) {
 	    fprintf (pf, "%d\n", (int)getpid());
 	    fclose (pf);
 	}
@@ -1742,7 +1790,7 @@ write_x_servers (GdmDisplay *d)
 	if (d->x_servers_order < 0)
 		d->x_servers_order = get_new_order (d);
 
-	fp = fopen (file, "w");
+	fp = gdm_safe_fopen_w (file);
 	if (fp == NULL) {
 		gdm_error ("Can't open %s for writing", file);
 		g_free (file);
@@ -2641,6 +2689,8 @@ static gboolean
 update_config (const char *key)
 {
 	struct stat statbuf;
+	VeConfig *cfg;
+
 	if (stat (GDM_CONFIG_FILE, &statbuf) == -1) {
 		/* if the file didn't exist before either */
 		if (config_file_mtime == 0)
@@ -2652,10 +2702,10 @@ update_config (const char *key)
 		config_file_mtime = statbuf.st_mtime;
 	}
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+	cfg = ve_config_new (GDM_CONFIG_FILE);
 
 	if (is_key (key, GDM_KEY_ALLOWROOT)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWROOT);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_ALLOWROOT);
 		if (ve_bool_equal (val, GdmAllowRoot))
 			goto update_config_ok;
 		GdmAllowRoot = val;
@@ -2664,7 +2714,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_ALLOWREMOTEROOT)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEROOT);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEROOT);
 		if (ve_bool_equal (val, GdmAllowRemoteRoot))
 			goto update_config_ok;
 		GdmAllowRemoteRoot = val;
@@ -2673,7 +2723,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_ALLOWREMOTEAUTOLOGIN)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEAUTOLOGIN);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEAUTOLOGIN);
 		if (ve_bool_equal (val, GdmAllowRemoteAutoLogin))
 			goto update_config_ok;
 		GdmAllowRemoteAutoLogin = val;
@@ -2682,7 +2732,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_SYSMENU)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_SYSMENU);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_SYSMENU);
 		if (ve_bool_equal (val, GdmSystemMenu))
 			goto update_config_ok;
 		GdmSystemMenu = val;
@@ -2691,7 +2741,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_CONFIG_AVAILABLE)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_CONFIG_AVAILABLE);
 		if (ve_bool_equal (val, GdmConfigAvailable))
 			goto update_config_ok;
 		GdmConfigAvailable = val;
@@ -2700,7 +2750,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_CHOOSER_BUTTON)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_CHOOSER_BUTTON);
 		if (ve_bool_equal (val, GdmChooserButton))
 			goto update_config_ok;
 		GdmChooserButton = val;
@@ -2709,7 +2759,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_RETRYDELAY)) {
-		int val = gnome_config_get_int (GDM_KEY_RETRYDELAY);
+		int val = ve_config_get_int (cfg, GDM_KEY_RETRYDELAY);
 		if (val == GdmRetryDelay)
 			goto update_config_ok;
 		GdmRetryDelay = val;
@@ -2718,7 +2768,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_DISALLOWTCP)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_DISALLOWTCP);
 		if (ve_bool_equal (val, GdmDisallowTCP))
 			goto update_config_ok;
 		GdmDisallowTCP = val;
@@ -2727,7 +2777,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_GREETER)) {
-		char *val = gnome_config_get_string (GDM_KEY_GREETER);
+		char *val = ve_config_get_string (cfg, GDM_KEY_GREETER);
 		if (strcmp (ve_sure_string (val), ve_sure_string (GdmGreeter)) == 0) {
 			g_free (val);
 			goto update_config_ok;
@@ -2739,7 +2789,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_REMOTEGREETER)) {
-		char *val = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
+		char *val = ve_config_get_string (cfg, GDM_KEY_REMOTEGREETER);
 		if (strcmp (ve_sure_string (val), ve_sure_string (GdmRemoteGreeter)) == 0) {
 			g_free (val);
 			goto update_config_ok;
@@ -2752,14 +2802,14 @@ update_config (const char *key)
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_TIMED_LOGIN) ||
 		   is_key (key, GDM_KEY_TIMED_LOGIN_ENABLE)) {
-		gboolean enable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
+		gboolean enable = ve_config_get_bool (cfg, GDM_KEY_TIMED_LOGIN_ENABLE);
 		char *val;
 
 		/* if not enabled, we just don't care */
 		if ( ! enable && ! GdmTimedLoginEnable)
 			goto update_config_ok;
 
-		val = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
+		val = ve_config_get_string (cfg, GDM_KEY_TIMED_LOGIN);
 		if (strcmp (ve_sure_string (val),
 			    ve_sure_string (GdmTimedLogin)) == 0 &&
 		    ve_bool_equal (enable, GdmTimedLoginEnable)) {
@@ -2781,7 +2831,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_TIMED_LOGIN_DELAY)) {
-		int val = gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+		int val = ve_config_get_int (cfg, GDM_KEY_TIMED_LOGIN_DELAY);
 		if (val == GdmTimedLoginDelay)
 			goto update_config_ok;
 		GdmTimedLoginDelay = val;
@@ -2790,7 +2840,7 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_XDMCP)) {
-		gboolean val = gnome_config_get_bool (GDM_KEY_XDMCP);
+		gboolean val = ve_config_get_bool (cfg, GDM_KEY_XDMCP);
 		if (ve_bool_equal (val, GdmXdmcp))
 			goto update_config_ok;
 		GdmXdmcp = val;
@@ -2804,16 +2854,16 @@ update_config (const char *key)
 
 		goto update_config_ok;
 	} else if (is_key (key, "xdmcp/PARAMETERS")) {
-		GdmDispPerHost = gnome_config_get_int (GDM_KEY_DISPERHOST);
-		GdmMaxPending = gnome_config_get_int (GDM_KEY_MAXPEND);
-		GdmMaxManageWait = gnome_config_get_int (GDM_KEY_MAXWAIT);
-		GdmMaxSessions = gnome_config_get_int (GDM_KEY_MAXSESS);
-		GdmIndirect = gnome_config_get_bool (GDM_KEY_INDIRECT);
-		GdmMaxIndirect = gnome_config_get_int (GDM_KEY_MAXINDIR);
-		GdmMaxIndirectWait = gnome_config_get_int (GDM_KEY_MAXINDWAIT);
-		GdmPingInterval = gnome_config_get_int (GDM_KEY_PINGINTERVAL);
+		GdmDispPerHost = ve_config_get_int (cfg, GDM_KEY_DISPERHOST);
+		GdmMaxPending = ve_config_get_int (cfg, GDM_KEY_MAXPEND);
+		GdmMaxManageWait = ve_config_get_int (cfg, GDM_KEY_MAXWAIT);
+		GdmMaxSessions = ve_config_get_int (cfg, GDM_KEY_MAXSESS);
+		GdmIndirect = ve_config_get_bool (cfg, GDM_KEY_INDIRECT);
+		GdmMaxIndirect = ve_config_get_int (cfg, GDM_KEY_MAXINDIR);
+		GdmMaxIndirectWait = ve_config_get_int (cfg, GDM_KEY_MAXINDWAIT);
+		GdmPingInterval = ve_config_get_int (cfg, GDM_KEY_PINGINTERVAL);
 	} else if (is_key (key, GDM_KEY_UDPPORT)) {
-		int val = gnome_config_get_int (GDM_KEY_UDPPORT);
+		int val = ve_config_get_int (cfg, GDM_KEY_UDPPORT);
 		if (GdmPort == val)
 			goto update_config_ok;
 		GdmPort = val;
@@ -2825,12 +2875,12 @@ update_config (const char *key)
 		}
 	}
 
-	gnome_config_pop_prefix ();
+	ve_config_destroy (cfg);
 	return FALSE;
 
 update_config_ok:
 
-	gnome_config_pop_prefix ();
+	ve_config_destroy (cfg);
 	return TRUE;
 }
 
