@@ -90,11 +90,6 @@ enum {
 #define GDM_SLANG      'R'
 #define GDM_RESET      'A'
 #define GDM_QUIT       'P'
-#if 0
-/* FIXME: Maybe just whack this */
-#define GDM_GNOMESESS  '?'
-#define GDM_SGNOMESESS '*'
-#endif
 /* Well these aren't as nice as above, oh well */
 #define GDM_STARTTIMER 's'
 #define GDM_STOPTIMER  'S'
@@ -149,10 +144,6 @@ enum {
 #define GDM_KEY_XKEEPSCRASHING "daemon/XKeepsCrashing=" EXPANDED_SYSCONFDIR "/gdm/XKeepsCrashing"
 #define GDM_KEY_REBOOT "daemon/RebootCommand=/usr/bin/reboot;/sbin/reboot;/sbin/shutdown -r now;/usr/sbin/shutdown -r now"
 #define GDM_KEY_ROOTPATH "daemon/RootPath=/sbin:/usr/sbin:/bin:/usr/bin:" X_CONF_PATH ":" EXPANDED_BINDIR
-#if 0
-/* FIXME: Maybe just whack this */
-#define GDM_KEY_GNOMEDEFAULTSESSION "daemon/GnomeDefaultSession=" EXPANDED_DATADIR "/gnome/default.session"
-#endif
 #define GDM_KEY_SERVAUTH "daemon/ServAuthDir=" EXPANDED_AUTHDIR
 #define GDM_KEY_SESSDIR "daemon/SessionDesktopDir=" EXPANDED_SYSCONFDIR "/dm/Sessions/"
 #define GDM_KEY_BASEXSESSION "daemon/BaseXsession=" EXPANDED_SYSCONFDIR "/gdm/Xsession"
@@ -223,7 +214,6 @@ enum {
 #define GDM_KEY_MINIMALUID "greeter/MinimalUID=100"
 #define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "nobody.png"
 #define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
-#define GDM_KEY_ICON "greeter/Icon=" EXPANDED_PIXMAPDIR "/gdm.png"
 #define GDM_KEY_LOCFILE "greeter/LocaleFile=" EXPANDED_LOCALEDIR "/locale.alias"
 #define GDM_KEY_LOGO "greeter/Logo=" EXPANDED_PIXMAPDIR "/gdm-foot-logo.png"
 #define GDM_KEY_QUIVER "greeter/Quiver=true"
@@ -262,20 +252,12 @@ enum {
 
 #define GDM_KEY_SERVERS "servers"
 
-#if 0
-/* FIXME: Maybe just whack this */
-#define GDM_KEY_SHOW_GNOME_CHOOSER "greeter/ShowGnomeChooserSession=true"
-#endif
 #define GDM_KEY_SHOW_GNOME_FAILSAFE "greeter/ShowGnomeFailsafeSession=true"
 #define GDM_KEY_SHOW_XTERM_FAILSAFE "greeter/ShowXtermFailsafeSession=true"
 #define GDM_KEY_SHOW_LAST_SESSION "greeter/ShowLastSession=true"
 
 #define GDM_SESSION_FAILSAFE_GNOME "GDM_Failsafe.GNOME"
 #define GDM_SESSION_FAILSAFE_XTERM "GDM_Failsafe.XTERM"
-#if 0
-/* FIXME: Maybe just whack this */
-#define GDM_SESSION_GNOME_CHOOSER "Gnome Chooser"
-#endif
 
 #define GDM_STANDARD "Standard"
 
@@ -445,7 +427,9 @@ void		gdm_final_cleanup	(void);
  * or gdmconfig or whatnot */
 
 /* The ones that pass a <slave pid> must be from a valid slave, and
- * the slave will be sent a SIGUSR2 */
+ * the slave will be sent a SIGUSR2.  Nowdays there is a pipe that is
+ * used from inside slaves, so those messages may stop being processed
+ * by the fifo at some point perhaps.  */
 /* The fifo protocol, used only by gdm internally */
 #define GDM_SOP_CHOSEN       "CHOSEN" /* <indirect id> <ip addr> */
 #define GDM_SOP_CHOSEN_LOCAL "CHOSEN_LOCAL" /* <slave pid> <hostname> */
