@@ -88,7 +88,7 @@ void gdm_desetuid (void);
 
 gboolean gdm_test_opt (const char *cmd, const char *help, const char *option);
 
-void gdm_close_all_descriptors (int from, int except);
+void gdm_close_all_descriptors (int from, int except, int except2);
 
 int gdm_open_dev_null (mode_t mode);
 
@@ -108,6 +108,22 @@ FILE * gdm_safe_fopen_ap (const char *file);
 
 void gdm_reset_limits (void);
 
+const char *gdm_root_user (void);
+
+#include <setjmp.h>
+
+/* stolen from xdm sources */
+#if defined(X_NOT_POSIX) || defined(__EMX__) || defined(__NetBSD__) && defined(__sparc__)
+#define Setjmp(e)	setjmp(e)
+#define Longjmp(e,v)	longjmp(e,v)
+#define Jmp_buf		jmp_buf
+#else
+#define Setjmp(e)   sigsetjmp(e,1)
+#define Longjmp(e,v)	siglongjmp(e,v)
+#define Jmp_buf		sigjmp_buf
+#endif
+
+
 #endif /* GDM_MISC_H */
 
 /* EOF */
