@@ -43,6 +43,47 @@ extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gint GdmRetryDelay;
 
+static char *selected_user = NULL;
+
+void
+gdm_verify_select_user (const char *user)
+{
+	g_free (selected_user);
+	if (ve_string_empty (user))
+		selected_user = NULL;
+	else
+		selected_user = g_strdup (user);
+}
+
+static void
+print_cant_auth_errbox (void)
+{
+	gboolean is_capslock = FALSE;
+	const char *basemsg;
+	char *msg;
+	char *ret;
+
+	ret = gdm_slave_greeter_ctl (GDM_QUERY_CAPSLOCK, "");
+	if ( ! ve_string_empty (ret))
+		is_capslock = TRUE;
+	g_free (ret);
+
+	basemsg = _("\nIncorrect username or password.  "
+		    "Letters must be typed in the correct "
+		    "case.");
+	if (is_capslock) {
+		msg = g_strconcat (basemsg, "  ",
+				   _("Please make sure the "
+				     "Caps Lock key is not "
+				     "enabled."),
+				   NULL);
+	} else {
+		msg = g_strdup (basemsg);
+	}
+	gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, msg);
+	g_free (msg);
+}
+
 /**
  * gdm_verify_user:
  * @username: Name of user or NULL if we should ask
@@ -58,7 +99,7 @@ extern gint GdmRetryDelay;
 gchar *
 gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboolean local) 
 {
-    gchar *login, *passwd, *ppasswd, *auth_errmsg;
+    gchar *login, *passwd, *ppasswd;
     struct passwd *pwent;
     struct spwd *sp;
 
@@ -66,20 +107,37 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    gdm_slave_greeter_ctl_no_ret (GDM_STARTTIMER, "");
 
     if (username == NULL) {
+authenticate_again:
 	    /* Ask for the user's login */
+	    gdm_verify_select_user (NULL);
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
-	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
+	    login = gdm_slave_greeter_ctl (GDM_PROMPT, _("Username:"));
 	    if (login == NULL ||
 		gdm_slave_greeter_check_interruption ()) {
-		    if (local)
-			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
-		    g_free (login);
-		    return NULL;
+		    if ( ! ve_string_empty (selected_user)) {
+			    /* user selected */
+			    g_free (login);
+			    login = selected_user;
+			    selected_user = NULL;
+		    } else {
+			    /* some other interruption */
+			    if (local)
+				    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+			    g_free (login);
+			    return NULL;
+		    }
 	    }
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
+
+	    if ( ! gdm_slave_check_user_wants_to_log_in (login)) {
+		    g_free (login);
+		    login = NULL;
+		    goto authenticate_again;
+	    }
     } else {
 	    login = g_strdup (username);
     }
+    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
 
     pwent = getpwnam (login);
 
@@ -88,21 +146,29 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     /* Lookup shadow password */
     sp = getspnam (login);
     
-    if (sp)
-	ppasswd = g_strdup (sp->sp_pwdp);
-    else
-	ppasswd = NULL;
+    /* Use shadow password when available */
+    if (sp != NULL) {
+	    ppasswd = g_strdup (sp->sp_pwdp);
+    } else {
+	    /* In case shadow password cannot be retrieved (when using NIS
+	       authentication for example), use standard passwd */
+	    if (pwent != NULL &&
+		pwent->pw_passwd != NULL)
+		    ppasswd = g_strdup (pwent->pw_passwd);
+	    else
+		    /* If no password can be retrieved, set it to NULL */
+		    ppasswd = NULL;
+    }
     
     endspent ();
 
     /* Request the user's password */
     if (pwent != NULL &&
-        (ve_string_empty (ppasswd) ||
-	 (local && gdm_is_a_no_password_user (login)))) {
+        ve_string_empty (ppasswd)) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
-	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
+	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password:"));
 	    if (passwd == NULL)
 		    passwd = g_strdup ("");
 	    if (gdm_slave_greeter_check_interruption ()) {
@@ -119,18 +185,11 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
-	    gdm_sleep_no_signal (GdmRetryDelay);
-	    gdm_error (_("Couldn't authenticate user"));
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password. "
-		       "Letters must be typed in the correct case. "  
-		       "Please make sure the Caps Lock key is not enabled."));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    sleep (GdmRetryDelay);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -140,17 +199,11 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    gdm_sleep_no_signal (GdmRetryDelay);
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password. "
-		       "Letters must be typed in the correct case. "  
-		       "Please make sure the Caps Lock key is not enabled."));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    sleep (GdmRetryDelay);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -216,10 +269,13 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
  */
 
 gboolean
-gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display) 
+gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
+		       char **new_login) 
 {
 	struct passwd *pwent;
 
+	*new_login = NULL;
+
 	pwent = getpwnam (login);
 	if (pwent == NULL) {
 		gdm_error (_("Cannot get passwd structure for %s"), login);
