@@ -23,6 +23,7 @@
 #include "greeter_item_clock.h"
 #include "greeter_item_pam.h"
 #include "greeter_item_ulist.h"
+#include "greeter_item_customlist.h"
 #include "greeter_item_capslock.h"
 #include "greeter_item_timed.h"
 #include "greeter_events.h"
@@ -40,10 +41,6 @@ gboolean GDM_IS_LOCAL = FALSE;
 char *GdmGraphicalTheme = NULL;
 char *GdmGraphicalThemeDir = NULL;
 int GdmXineramaScreen = 0;
-#if 0
-/* FIXME: maybe just whack this */
-gboolean GdmShowGnomeChooserSession = FALSE;
-#endif
 gboolean GdmShowGnomeFailsafeSession = FALSE;
 gboolean GdmShowXtermFailsafeSession = FALSE;
 gboolean GdmShowLastSession = FALSE;
@@ -71,6 +68,7 @@ int GdmMinimalUID;
 gboolean GdmAllowRoot;
 gboolean GdmAllowRemoteRoot;
 gchar *GdmWelcome;
+gchar *GdmServAuthDir;
 
 gboolean GdmUseCirclesInEntry = FALSE;
 
@@ -125,10 +123,6 @@ greeter_parse_config (void)
 
     GdmShowXtermFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_XTERM_FAILSAFE);
     GdmShowGnomeFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_GNOME_FAILSAFE);
-#if 0
-    /* FIXME: maybe just whack this */
-    GdmShowGnomeChooserSession = ve_config_get_bool (config, GDM_KEY_SHOW_GNOME_CHOOSER);
-#endif
     GdmShowLastSession = ve_config_get_bool (config, GDM_KEY_SHOW_LAST_SESSION);
     GdmSessionDir = ve_config_get_string (config, GDM_KEY_SESSDIR);
     GdmDefaultSession = ve_config_get_string (config, GDM_KEY_DEFAULTSESSION);
@@ -141,6 +135,7 @@ greeter_parse_config (void)
     GdmSuspend = ve_config_get_string (config, GDM_KEY_SUSPEND);
     GdmConfigurator = ve_config_get_string (config, GDM_KEY_CONFIGURATOR);
     GdmGtkRC = ve_config_get_string (config, GDM_KEY_GTKRC);
+    GdmServAuthDir = ve_config_get_string (config, GDM_KEY_SERVAUTH);
 
     GdmWelcome = ve_config_get_translated_string (config, greeter_Welcome_key);
     /* A hack! */
@@ -437,47 +432,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	_exit (EXIT_SUCCESS);
 	break;
 
-#if 0
-	/* FIXME: Maybe whack this */
-    case GDM_GNOMESESS:
-	{
-		char *sess;
-		GString *str = g_string_new (NULL);
-
-		do {
-			g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL);
-			buf[len-1] = '\0';
-			tmp = ve_locale_to_utf8 (buf);
-			g_string_append (str, tmp);
-			g_free (tmp);
-		} while (len == PIPE_SIZE-1);
-
-
-		sess = greeter_get_gnome_session (str->str);
-
-		g_string_free (str, TRUE);
-
-		tmp = ve_locale_from_utf8 (sess);
-		printf ("%c%s\n", STX, tmp);
-		fflush (stdout);
-		g_free (tmp);
-
-		g_free (sess);
-	}
-	break;
-
-    case GDM_SGNOMESESS:
-	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
-
-	if (greeter_save_gnome_session ())
-	    printf ("%cY\n", STX);
-	else
-	    printf ("%c\n", STX);
-	fflush (stdout);
-
-	break;
-#endif
-
     case GDM_STARTTIMER:
 	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
@@ -620,6 +574,9 @@ greeter_setup_items (void)
 					 NULL);
   greeter_item_system_setup ();
   greeter_item_session_setup ();
+
+  /* Setup the custom widgets */
+  greeter_item_customlist_setup ();
 }
 
 enum {
@@ -867,8 +824,6 @@ string_same (VeConfig *config, const char *cur, const char *key)
 		g_free (val);
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "string not same: cur '%s' val '%s' key '%s'",
-			cur, val, key);
 		g_free (val);
 		return FALSE;
 	}
@@ -881,8 +836,6 @@ bool_same (VeConfig *config, gboolean cur, const char *key)
 	if (ve_bool_equal (cur, val)) {
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "bool not same: cur '%d' val '%d' key '%s'",
-			cur, val, key);
 		return FALSE;
 	}
 }
@@ -894,8 +847,6 @@ int_same (VeConfig *config, int cur, const char *key)
 	if (cur == val) {
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "int not same: cur '%d' val '%d' key '%s'",
-			cur, val, key);
 		return FALSE;
 	}
 }
@@ -940,12 +891,6 @@ greeter_reread_config (int sig, gpointer data)
 	     ! bool_same (config,
 			  GdmShowGnomeFailsafeSession,
 			  GDM_KEY_SHOW_GNOME_FAILSAFE) ||
-#if 0
-	     /* FIXME: maybe just whack this */
-	     ! bool_same (config,
-			  GdmShowGnomeChooserSession,
-			  GDM_KEY_SHOW_GNOME_CHOOSER) ||
-#endif
 	     ! string_same (config,
 			    GdmSessionDir,
 			    GDM_KEY_SESSDIR) ||
@@ -979,9 +924,6 @@ greeter_reread_config (int sig, gpointer data)
 	     ! int_same (config,
 			 GdmTimedLoginDelay,
 			 GDM_KEY_TIMED_LOGIN_DELAY)) {
-		syslog (LOG_ERR, "something not same: "
-			"theme '%s' theme_dir '%s'",
-			theme, theme_dir);
 		/* Set busy cursor */
 		greeter_setup_cursor (GDK_WATCH);
 
@@ -1281,7 +1223,7 @@ main (int argc, char *argv[])
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_OK,
 				    s,
-				    /* avoid warning */ "%s", "");
+				    "%s", (error && error->message) ? error->message : "");
 	g_free (s);
     
         gtk_widget_show_all (dialog);
@@ -1294,6 +1236,9 @@ main (int argc, char *argv[])
         exit(1);
       }
 
+  if (error)
+    g_clear_error (&error);
+
   /* Try circles.xml */
   if (root == NULL)
     {
@@ -1307,7 +1252,7 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
-  if (greeter_lookup_id ("user-pw-entry") == NULL)
+  if (root != NULL && greeter_lookup_id ("user-pw-entry") == NULL)
     {
       GtkWidget *dialog;
 
